@@ -4,21 +4,22 @@ import hmac
 import json
 import time
 import urllib
-import requests
 from copy import copy
 from enum import Enum
 from ssl import SSLError
 from threading import Lock
 from datetime import datetime, timedelta
 from typing import Dict, Any, Tuple, List
-from strategy_api.event.engine import EVENT_TIMER, Event
+from strategy_api.event.engine import EVENT_TIMER, Event, EventEngine
 from strategy_api.tm_api.api.rest.rest_client import RestClient, Request, Response
 from strategy_api.tm_api.api.websocket.websocket_client import WebsocketClient
 from strategy_api.tm_api.base import BaseGateway
 from strategy_api.tm_api.object import TickData, BarData, Interval, OrderRequest, OrderData, Direction, OrderType, \
-    Offset, Status, CancelRequest, HistoryRequest, PositionSide
+    Offset, Status, CancelRequest, HistoryRequest, PositionSide, DataType, Product, Exchange
 
 # http 基础地址
+from strategy_api.tm_api.tools import get_order_type, symbol_deal
+
 F_REST_HOST: str = "https://fapi.binance.com"
 
 # ws 基础地址
@@ -86,10 +87,13 @@ class BinanceFutureUsdtGateway(BaseGateway):
         "proxy_host": "",
         "proxy_port": 0,
     }
+    exchange: Exchange = Exchange.BINANCE
+    product: Product = Product.U_FUTURES
+    symbolMap: dict = dict()
 
-    def __init__(self) -> None:
+    def __init__(self, event_engine: EventEngine, order_switch: bool = False, bar_switch: bool = False, tick_switch: bool = False) -> None:
         """init"""
-        super().__init__()
+        super().__init__(event_engine, order_switch, bar_switch, tick_switch)
         self.trade_ws_api: "BinanceUsdtTradeWebsocketApi" = BinanceUsdtTradeWebsocketApi(self)
         self.market_ws_api: "BinanceUsdtDataWebsocketApi" = BinanceUsdtDataWebsocketApi(self)
         self.rest_api: "BinanceUsdtRestApi" = BinanceUsdtRestApi(self)
@@ -123,25 +127,9 @@ class BinanceFutureUsdtGateway(BaseGateway):
             self.rest_api.query_time()
             self.get_server_time_interval = 0
 
-    def get_order_type(self, maker: bool = False, stop_loss: bool = False,
-                       stop_profit: bool = False) -> OrderType:
-        if maker:
-            # 发送限价单
-            order_type = OrderType.LIMIT
-        else:
-            if stop_profit:
-                # 发送市价止盈
-                order_type = OrderType.TAKE_PROFIT_MARKET
-            elif stop_loss:
-                # 发送市价止损
-                order_type = OrderType.STOP_MARKET
-            else:
-                # 发送普通市价单
-                order_type = OrderType.MARKET
-        return order_type
-
     # 多头买进
     def buy(self,
+            orderid: str,
             symbol: str,
             volume: float,  # 数量
             price: float = 0,  # 价格
@@ -149,21 +137,25 @@ class BinanceFutureUsdtGateway(BaseGateway):
             stop_loss: bool = False,  # 止损
             stop_profit: bool = False,  # 止盈
             position_side: PositionSide = PositionSide.ONEWAY,
-            ) -> str:
+            ):
+        new_symbol = symbol_deal(symbol, self.exchange)
+        self.symbolMap[new_symbol] = symbol
 
         original_req: OrderRequest = OrderRequest(
-            symbol=symbol,
+            symbol=new_symbol,
             direction=Direction.LONG,
             offset=Offset.OPEN,
-            type=self.get_order_type(maker, stop_loss, stop_profit),
+            type=get_order_type(maker, stop_loss, stop_profit),
             price=price,
             volume=volume,
             positionSide=position_side,
+            exchange=self.exchange
         )
-        return self.send_order(original_req)
+        self.send_order(original_req, orderid)
 
     # 多头卖出
     def sell(self,
+             orderid: str,
              symbol: str,
              volume: float,  # 数量
              price: float = 0,  # 价格
@@ -171,20 +163,25 @@ class BinanceFutureUsdtGateway(BaseGateway):
              stop_loss: bool = False,  # 止损
              stop_profit: bool = False,  # 止盈
              position_side: PositionSide = PositionSide.ONEWAY,
-             ) -> str:
+             ):
+        new_symbol = symbol_deal(symbol, self.exchange)
+        self.symbolMap[new_symbol] = symbol
+
         original_req: OrderRequest = OrderRequest(
-            symbol=symbol,
+            symbol=new_symbol,
             direction=Direction.SHORT,
             offset=Offset.CLOSE,
-            type=self.get_order_type(maker, stop_loss, stop_profit),
+            type=get_order_type(maker, stop_loss, stop_profit),
             price=price,
             volume=volume,
             positionSide=position_side,
+            exchange=self.exchange
         )
-        return self.send_order(original_req)
+        self.send_order(original_req, orderid)
 
     # 空头买进
     def short(self,
+              orderid: str,
               symbol: str,
               volume: float,  # 数量
               price: float = 0,  # 价格
@@ -192,20 +189,25 @@ class BinanceFutureUsdtGateway(BaseGateway):
               stop_loss: bool = False,  # 止损
               stop_profit: bool = False,  # 止盈
               position_side: PositionSide = PositionSide.ONEWAY,
-              ) -> str:
+              ):
+        new_symbol = symbol_deal(symbol, self.exchange)
+        self.symbolMap[new_symbol] = symbol
+
         original_req: OrderRequest = OrderRequest(
-            symbol=symbol,
+            symbol=new_symbol,
             direction=Direction.SHORT,
             offset=Offset.OPEN,
-            type=self.get_order_type(maker, stop_loss, stop_profit),
+            type=get_order_type(maker, stop_loss, stop_profit),
             price=price,
             volume=volume,
             positionSide=position_side,
+            exchange=self.exchange
         )
-        return self.send_order(original_req)
+        self.send_order(original_req, orderid)
 
     # 空头卖出
     def cover(self,
+              orderid: str,
               symbol: str,
               volume: float,  # 数量
               price: float = 0,  # 价格
@@ -213,32 +215,43 @@ class BinanceFutureUsdtGateway(BaseGateway):
               stop_loss: bool = False,  # 止损
               stop_profit: bool = False,  # 止盈
               position_side: PositionSide = PositionSide.ONEWAY,
-              ) -> str:
+              ):
+        new_symbol = symbol_deal(symbol, self.exchange)
+        self.symbolMap[new_symbol] = symbol
+
         original_req: OrderRequest = OrderRequest(
-            symbol=symbol,
+            symbol=new_symbol,
             direction=Direction.LONG,
             offset=Offset.CLOSE,
-            type=self.get_order_type(maker, stop_loss, stop_profit),
+            type=get_order_type(maker, stop_loss, stop_profit),
             price=price,
             volume=volume,
             positionSide=position_side,
+            exchange=self.exchange
         )
-        return self.send_order(original_req)
+        self.send_order(original_req, orderid)
 
     # 发送订单
-    def send_order(self, req: OrderRequest) -> str:
-        return self.rest_api.send_order(req)
+    def send_order(self, req: OrderRequest, orderid: str):
+        self.rest_api.send_order(req, orderid)
 
     # 撤销订单
     def cancel_order(self, orderid: str, symbol: str) -> None:
+        new_symbol = symbol_deal(symbol, self.exchange)
+        self.symbolMap[new_symbol] = symbol
+
         req: CancelRequest = CancelRequest(
             orderid=orderid,
-            symbol=symbol
+            symbol=new_symbol,
+            exchange=self.exchange
         )
         self.rest_api.cancel_order(req)
 
     # 获取历史K线
     def query_history(self, symbol: str, interval: Interval, hour: int = 0, minutes: int = 0) -> List[BarData]:
+        new_symbol = symbol_deal(symbol, self.exchange)
+        self.symbolMap[new_symbol] = symbol
+
         end: datetime = datetime.now()
         if hour:
             start: datetime = end - timedelta(hours=hour)
@@ -249,16 +262,29 @@ class BinanceFutureUsdtGateway(BaseGateway):
             return []
 
         req: HistoryRequest = HistoryRequest(
-            symbol=symbol,
+            symbol=new_symbol,
             start=start,
             end=end,
-            interval=interval
+            interval=interval,
+            exchange=self.exchange
         )
         return self.rest_api.query_history(req)
 
     # 订阅数据
-    def subscribe(self, symbol: str, interval: Interval = None) -> None:
-        self.market_ws_api.subscribe(symbol, interval)
+    def subscribe(self, symbol: str, data_type: DataType, interval: Interval = None) -> None:
+        new_symbol = symbol_deal(symbol, self.exchange)
+        self.symbolMap[new_symbol] = symbol
+
+        if data_type == DataType.TICK:
+            self.market_ws_api.subscribe(new_symbol)
+        elif data_type == DataType.BAR:
+            if interval:
+                self.market_ws_api.subscribe(new_symbol, interval)
+            else:
+                print(f"订阅失败，k 线数据未定义间隔时间")
+        else:
+            print(f"订阅失败，未知数据类型: {data_type}")
+
 
     # 关闭
     def close(self) -> None:
@@ -355,7 +381,7 @@ class BinanceUsdtRestApi(RestClient):
 
         # 获取服务器时间
         self.query_time()
-        # 保持用户 ws 的链接
+        # 保持用户 ws 的链接：订单回调
         self.start_user_stream()
 
     # 查询服务器时间
@@ -382,9 +408,12 @@ class BinanceUsdtRestApi(RestClient):
             self.order_count += 1
             return self.order_count
 
-    # 发送订单
-    def send_order(self, req: OrderRequest) -> str:
+    def get_order_id(self) -> str:
         orderid: str = "x-cLbi5uMH" + str(self.connect_time + self._new_order_id())
+        return orderid
+
+    # 发送订单
+    def send_order(self, req: OrderRequest, orderid: str):
 
         order: OrderData = req.create_order_data(
             orderid,
@@ -406,7 +435,8 @@ class BinanceUsdtRestApi(RestClient):
             if req.offset == Offset.CLOSE:
                 params['reduceOnly'] = True
         else:
-            if (req.direction == Direction.SHORT and req.offset == Offset.OPEN) or (req.direction == Direction.LONG and req.offset == Offset.CLOSE):
+            if (req.direction == Direction.SHORT and req.offset == Offset.OPEN) or (
+                    req.direction == Direction.LONG and req.offset == Offset.CLOSE):
                 position_side = "SHORT"
             else:
                 position_side = "LONG"
@@ -449,8 +479,6 @@ class BinanceUsdtRestApi(RestClient):
             on_failed=self.on_send_order_failed
         )
 
-        return order.orderid
-
     # 撤销订单
     def cancel_order(self, req: CancelRequest) -> None:
         """cancel order"""
@@ -518,7 +546,7 @@ class BinanceUsdtRestApi(RestClient):
 
                 for row in data:
                     bar: BarData = BarData(
-                        symbol=req.symbol,
+                        symbol=self.gateway.symbolMap[req.symbol],
                         datetime=generate_datetime(row[0]),
                         endTime=generate_datetime(row[6]),
                         interval=req.interval,
@@ -527,7 +555,8 @@ class BinanceUsdtRestApi(RestClient):
                         open_price=float(row[1]),
                         high_price=float(row[2]),
                         low_price=float(row[3]),
-                        close_price=float(row[4])
+                        close_price=float(row[4]),
+                        exchange=self.gateway.exchange
                     )
                     buf.append(bar)
 
@@ -649,19 +678,7 @@ class BinanceUsdtRestApi(RestClient):
 
     # 发送订单成功回调
     def on_send_order(self, data: dict, request: Request) -> None:
-        if request.extra:
-            order: OrderData = copy(request.extra)
-            traded = float(data.get('executedQty', '0'))
-            traded_price = float(data.get('avgPrice', '0'))
-            price = float(data["price"])
-            if price <= 0 < traded_price:
-                price = traded_price
-
-            order.traded = traded
-            order.traded_price = traded_price
-            order.price = price
-            order.status = STATUS_BINANCES2VT.get(data.get('status'), Status.NOTTRADED)
-            self.gateway.on_order(order)
+        pass
 
     # 发送订单失败回调
     def on_send_order_failed(self, status_code: int, request: Request) -> None:
@@ -818,11 +835,14 @@ class BinanceUsdtDataWebsocketApi(WebsocketClient):
         self.start()
 
     def on_connected(self) -> None:
-        if self.ticks:
-            channels = []
-            for symbol in self.ticks.keys():
-                channels.append(f"{symbol}@aggTrade")
+        channels = []
+        for target in self.bars:
+            channels.append(target)
 
+        for target in self.ticks:
+            channels.append(target)
+
+        if channels:
             req: dict = {
                 "method": "SUBSCRIBE",
                 "params": channels,
@@ -830,41 +850,44 @@ class BinanceUsdtDataWebsocketApi(WebsocketClient):
             }
             self.send_packet(req)
 
-    def subscribe(self, symbol: str, interval: Interval) -> None:
+    def subscribe(self, symbol: str, interval: Interval = None) -> None:
         """订阅数据"""
         # 如果间隔参数不为 None, 那么就是订阅K线数据
         if interval:
+            target = f"{symbol.lower()}@kline_{interval.value}"
             # 订阅 K线 流程
-            if symbol in self.bars:
+            if target in self.bars:
                 return
             self.reqid += 1
             bar: BarData = BarData(
                 symbol=symbol,
                 datetime=datetime.now(),
                 endTime=datetime.now(),
-                interval=interval
+                interval=interval,
+                exchange=self.gateway.exchange,
+                product=self.gateway.product
             )
-            self.bars[symbol.lower() + f"{interval.value}"] = bar
-            channels = [
-                f"{symbol.lower()}@kline_{interval.value}"
-            ]
+            self.bars[target] = bar
+
         else:
+            target = f"{symbol.lower()}@aggTrade"
+
             # 订阅tick 数据流程
             if symbol in self.ticks:
                 return
             self.reqid += 1
             tick: TickData = TickData(
                 symbol=symbol,
-                datetime=datetime.now()
+                datetime=datetime.now(),
+                exchange=self.gateway.exchange,
+                product=self.gateway.product
             )
-            self.ticks[symbol.lower()] = tick
-            channels = [
-                f"{symbol.lower()}@aggTrade"
-            ]
+            self.ticks[target] = tick
+
 
         req: dict = {
             "method": "SUBSCRIBE",
-            "params": channels,
+            "params": [target],
             "id": self.reqid
         }
         self.send_packet(req)
@@ -877,8 +900,8 @@ class BinanceUsdtDataWebsocketApi(WebsocketClient):
         data: dict = packet["data"]
 
         symbol, channel = stream.split("@")
-        bar: BarData = self.bars.get(symbol + f"{channel.split('_')[-1]}")
-        tick: TickData = self.ticks.get(symbol)
+        bar: BarData = self.bars.get(stream)
+        tick: TickData = self.ticks.get(stream)
 
         if channel.startswith("kline_"):
             data = data['k']
